"""Module containing models for bird song detection."""
import os
import numpy as np

from tensorflow import keras
from tensorflow.keras import layers
from tensorflow import math


from microfaune.audio import load_wav, create_spec

RNN_WEIGHTS_FILE = os.path.abspath(
    os.path.join(os.path.dirname(__file__),
                 "data/model_weights_tf2-20200912_173814.h5"))


class RNNDetector:
    """Class wrapping a rnn model

    """
    def __init__(self, weights_file=RNN_WEIGHTS_FILE):
        """Initialization function"""
        self.weights_file = weights_file
        self._model = None

    @property
    def model(self):
        """Tensorflow Keras like model"""
        if self._model is None:
            self._model = self.create_model()
            self._model.load_weights(self.weights_file)
        return self._model

    def create_model(self):
        """Create RNN model."""
        n_filter = 64

        spec = layers.Input(shape=[None, 40, 1], dtype=np.float32)
        x = layers.Conv2D(n_filter, (3, 3), padding="same",
                          activation=None)(spec)
        x = layers.BatchNormalization(momentum=0.95)(x)
        x = layers.ReLU()(x)
        x = layers.Conv2D(n_filter, (3, 3), padding="same",
                          activation=None)(x)
        x = layers.BatchNormalization(momentum=0.95)(x)
        x = layers.ReLU()(x)
        x = layers.MaxPool2D((1, 2))(x)

        x = layers.Conv2D(n_filter, (3, 3), padding="same",
                          activation=None)(x)
        x = layers.BatchNormalization(momentum=0.95)(x)
        x = layers.ReLU()(x)
        x = layers.Conv2D(n_filter, (3, 3), padding="same",
                          activation=None)(x)
        x = layers.BatchNormalization(momentum=0.95)(x)
        x = layers.ReLU()(x)
        x = layers.MaxPool2D((1, 2))(x)

        x = layers.Conv2D(n_filter, (3, 3), padding="same",
                          activation=None)(x)
        x = layers.BatchNormalization(momentum=0.95)(x)
        x = layers.ReLU()(x)
        x = layers.Conv2D(n_filter, (3, 3), padding="same",
                          activation=None)(x)
        x = layers.BatchNormalization(momentum=0.95)(x)
        x = layers.ReLU()(x)
        x = layers.MaxPool2D((1, 2))(x)

        x = math.reduce_max(x, axis=-2)

        x = layers.Bidirectional(layers.GRU(64, return_sequences=True))(x)
        x = layers.Bidirectional(layers.GRU(64, return_sequences=True))(x)

        x = layers.TimeDistributed(layers.Dense(64, activation="sigmoid"))(x)
        local_pred = layers.TimeDistributed(
            layers.Dense(1, activation="sigmoid"))(x)
        pred = math.reduce_max(local_pred, axis=-2)
        return keras.Model(inputs=spec, outputs=[pred, local_pred])

    def compute_features(self, audio_signals):
        """Compute features on audio signals.

        Parameters
        ----------
        audio_signals: list
            Audio signals of possibly various lengths.

        Returns
        -------
        X: list
            Features for each audio signal
        """
        X = []
        for data in audio_signals:

            x = create_spec(data, fs=44100, n_mels=40, n_fft=2048,
                            hop_len=1024).transpose()
            X.append(x[..., np.newaxis].astype(np.float32)/255)
        return X

    def predict_on_wav(self, wav_file):
        """Detect bird presence in wav file.

        Parameters
        ----------
        wav_file: str
            wav file path.

        Returns
        -------
        score: float
            Prediction score of the classifier on the whole sequence
        local_score: array-like
            Time step prediction score
        """
        fs, data = load_wav(wav_file)
        X = self.compute_features([data])
        scores, local_scores = self.predict(np.array(X))
        return scores[0], local_scores[0]

    def predict(self, X):
        """Predict bird presence on spectograms.

        Parameters
        ----------
        X: array-like
            List of features on which to run the model.

        Returns
        -------
        scores: array-like
            Prediction scores of the classifier on each audio signal
        local_scores: array-like
            Step-by-step  prediction scores for each audio signal
        """
        scores = []
        local_scores = []
        for x in X:
            s, local_s = self.model.predict(x[np.newaxis, ...])
            scores.append(s[0])
            local_scores.append(local_s.flatten())
        scores = np.array(s)
        return scores, local_scores

    def free_mem(self):
        """Release GPU memory."""
        self._model = None

if __name__ == '__main__' :
    detector = RNNDetector()
    global_score, local_score = detector.predict_on_wav(os.path.abspath(os.path.join(os.path.dirname(__file__), "media/SWIFT_20190723_050006.wav"))) # NB: Check that loaded wav file actually exists on your disk
    print(f"Golbal score: {global_score}  -  Localscore: {local_score}")




import numpy as np
import json
from scipy import interpolate
from scipy.io import wavfile
import random

from microfaune import audio, plot


def read_json_file(json_file_path):
    """ Read json file with labels.

                Parameters
                ----------
                json_file_path : str
                    Path of json file.

                Returns:
                -------
                data_dict : list
                    List of labels, each label is a dictionary item with entries 'id', 'start', 'end', 'annotation'
    """
    with open(json_file_path) as json_data:
        data_dict = json.load(json_data)
    return data_dict


def number_labels(json_file_path):
    """ Count labels in json file.

            Parameters
            ----------
            json_file_path : str
                Path of json file.

            Returns:
            -------
            nb_labels : int
                Number of labels in json file
        """
    data_dict = read_json_file(json_file_path)
    nb_labels = len(data_dict)
    return nb_labels


def prop_labeled(json_file_path, audio_file_path):
    """ Compute proportion of the ratio of labels duration compared to total duration of audio

        Parameters
        ----------
        json_file_path : str
            Path of json file.
        audio_file_path : str
            Path of audio file.

        Returns:
        -------
        ratio : float
            Ratio of duration of labeled segments and total duration of audio.
    """

    fs, data = audio.load_audio(audio_file_path)
    total_duration = len(data) / fs

    data_dict = read_json_file(json_file_path)

    bird_song_duration = 0

    for label in data_dict:
        bird_song_duration += label['end'] - label['start']
    ratio = round(bird_song_duration / total_duration, 2)

    return ratio


def charac_function_audio(json_file_path, audio_file_path):
    """ Derive the characteristic function from the labels in json file
            Parameters
            ----------
            json_file_path : str
                Path of json file.
            audio_file_path : str
                Path of audio file.

            Returns:
            -------
            charac_func : numpy array (nb bites in audio,1)
                Characteristic function derived on audio time scale, equal to 1 on labeled segments, 0 elsewhere
    """
    fs, data = audio.load_audio(audio_file_path)
    charac_func = np.zeros((len(data), 1))

    data_dict = read_json_file(json_file_path)

    for label in data_dict:
        indx_start = int(label['start'] * fs)
        indx_end = int(label['end'] * fs)
        charac_func[indx_start:indx_end + 1, 0] = 1

    return charac_func


def charac_function_spec(audio_file_path, window_length, overlap, charac_func_audio):
    """ Derive the characteristic function from the labels in json file
            Parameters
            ----------
            audio_file_path : str
                Path of audio file.
            window_length : float
                Length of the FFT window in seconds.
            overlap : float
                Overlap of the FFT windows.
            charac_func_audio : numpy array (nb of samples in audio file,1)
                Characteristic function derived in audio time scale, equal to 1 on labeled segments, 0 elsewhere

            Returns:
            -------
            charac_func_spec : numpy array (nb of samples in spectrogram,1)
                Characteristic function derived in spectrogram time scale, equal to 1 on labeled segments, 0 elsewhere
        """

    fs, data = audio.load_audio(audio_file_path)
    duration = len(data) / fs
    size_spec = 2 + int(duration / (window_length * (1 - overlap)))
    size_audio = len(charac_func_audio)
    regroup_factor = int(size_audio / size_spec)

    charac_func_spec = np.zeros((size_spec, 1))

    for i in range(size_spec):
        label_local = np.mean(charac_func_audio[i * regroup_factor: (i + 1) * regroup_factor])
        if label_local > 0.5:
            charac_func_spec[i] = 1

    return charac_func_spec


def charac_function_fs(old_fs, new_fs, charac_func):
    """ Convert the scale of a characteristic function from spec time scale to another time scale with the desired sampling rate
        Parameters
        ----------
            old_fs: int
                Sampling frequency in Hz on which charac_func is derived.
            new_fs: int
                Sampling frequency in Hz to which charac_func needs to be  converted.
            charac_func: numpy array (nb of samples, 1)
                Characteristic function with sampling rate old_fs

        Returns:
        -------
            charac_func_new_fs : numpy array (nb of samples, 1)
                Characteristic function with sampling rate new_fs
        """

    duration = len(charac_func) / old_fs

    t_old = np.linspace(0, duration, num = len(charac_func))
    t_new = np.linspace(0, duration, num = duration*new_fs)
    f = interpolate.interp1d(t_old, charac_func[:,0])
    charac_func_fs = np.zeros((int(duration*new_fs), 1))
    charac_func_fs[:,0] = f(t_new)

    return charac_func_fs


def charac_function_spec_fs(fs, window_length, overlap, charac_func_spec):
    """ Convert the scale of a characteristic function from spec time scale to another time scale with the desired sampling rate
        Parameters
        ----------
            fs: int
                Sampling frequency in Hz.
            window_length : float
                Length of the FFT window in seconds.
            overlap : float
                Overlap of the FFT windows.
            charac_func_spec: numpy array (nb of samples in spectrogram, 1)
                Characteristic function derived in spectrogram time scale, equal to 1 on labeled segments, 0 elsewhere

        Returns:
        -------
            charac_func_fs : numpy array (nb of samples, 1)
                Characteristic function derived with the desired sampling rate
        """

    dt_spec = window_length * (1 - overlap)
    fs_spec = 1./dt_spec
    charac_func_fs = charac_function_fs(fs_spec, fs, charac_func_spec)

    return charac_func_fs


def create_wav_with_label(fs, charac_func_fs, file_path):
    """ Create a false wav file with characteristic function.
        Used in Audacity to plot spectrogram and label at the same time
        Parameters
        ----------
                fs: int
                    Sampling frequency in Hz.
                charac_func_fs : numpy array (fs*duration,1)
                    Characteristic function derived with the desired sampling rate
                file_path : str
                    Path of the wav file path saved
    """
    wavfile.write(file_path, fs, charac_func_fs[:, 0])

    return None


def create_label_json(path, labels=[], start_time=None):
    """ Write a json file from a list of labels
    Parameters
    ----------
        path: str
            Path of the future json file.
        labels: list
            List of labels, each label is a dictionary with keys 'id', 'start', 'end' and 'annotation'
        start_time: float
            Start time of audio extract in seconds
    """
    data_dict = []

    for label in labels:
        data_dict.append({
            'id': 'none',
            'start': label['start'] - start_time,
            'end': label['end'] - start_time,
            'annotation': 'bird'
        })

    with open(path, 'w') as outfile:
        json.dump(data_dict, outfile)

    return None


def extract_labels(json_path, start_time, duration):
    """ Extract the labels of the audio extract from the json creating at labeling
        Parameters
        ----------
            json_path: str
                Path of the json file.
            start_time: float
                Start time of audio extract in seconds
            duration: float
                Duration of audio extract in seconds
        Returns
        -------
            labels: list
                List of labelson the audio extract, each label is a dictionary with keys 'id', 'start', 'end' and 'annotation'
        """
    data_dict = read_json_file(json_path)
    labels = []

    for label in data_dict:

        start_label = label['start']
        end_label = label['end']

        if start_time < start_label:
            if start_time + duration > end_label:
                labels.append({
                    'start': max(start_time, start_label),
                    'end': min(start_time + duration, end_label)
                })
        else:
            if end_label > start_time:
                labels.append({
                    'start': max(start_time, start_label),
                    'end': min(start_time + duration, end_label)
                })

    return labels


def extract_audio(wav_folder_path, json_folder_path, wav_file, json_file, path_database, fs_filter, duration_extract, threshold,
                  nbre_extracts_pos, nbre_extracts_neg, max_counts):
    """ Extract the labels of the audio extract from the json creating at labeling
        Parameters
        ----------
            wav_folder_path: str
                Path of wav audio files
			json_folder_path: str
                Path of wav json files
            wav_file: str
                Name of the wav file
            json_file: str
                Name of the json file
            path_database: str
                Path used to save database
            fs_filter: int
                Sampling rate of the filter in Hz
            duration_extract: float
                Duration of audio to extract in seconds
            threshold: float
                Threshold for the proportion of labelised segments to avoid to have very short bird song in positive audio
            nbre_extracts_pos:
                Number of audio with bird songs to extract
            nbre_extracts_neg:
                Number of audio without bird songs to extract
            max_counts:
                Maximum try for random extraction
    """
    audio_fs, audio_data = wavfile.read(wav_folder_path + wav_file)

    duration_audio = len(audio_data) / audio_fs
    charac_audio = charac_function_audio(json_folder_path + json_file, wav_folder_path + wav_file)
    charac_audio = charac_function_fs(audio_fs, fs_filter, charac_audio)

    positive_extracts = 0
    negative_extracts = 0
    counts = 0

    test_label = np.where(charac_audio > 0)
    prop_labelised = len(test_label[0]) / len(charac_audio)
    # if no bird label in audio no positive audio extracted
    if len(test_label[0]) == 0:
        positive_extracts = nbre_extracts_pos
    # if there are too many labels no negative audio extracted
    elif duration_audio * (1 - prop_labelised) < duration_extract:
        negative_extracts = nbre_extracts_neg

    while (positive_extracts < nbre_extracts_pos or negative_extracts < nbre_extracts_neg) and counts < max_counts:

        start_time = round(random.random() * (duration_audio - duration_extract), 2)

        charac_filter = np.zeros((len(charac_audio), 1))
        indx_start = int(start_time * fs_filter)
        indx_end = int((duration_extract + start_time) * fs_filter) + 1
        charac_filter[indx_start:indx_end] = 1

        apply_filter = charac_filter * charac_audio

        labeled_segment = np.where(apply_filter > 0)
        labeled_prop = len(labeled_segment[0]) / (duration_extract * fs_filter)

        if labeled_prop > threshold and positive_extracts < nbre_extracts_pos:
            path_wav = path_database + 'positive/' + wav_file[:-4] + '_pos_t0_'+ str(start_time) + '.wav'
            indx_start = int(start_time * audio_fs)
            indx_end = int((duration_extract + start_time) * audio_fs) + 1
            wavfile.write(path_wav, audio_fs, audio_data[indx_start:indx_end])

            path_json = path_database + 'json/' + wav_file[:-4] + '_pos_t0_'+ str(start_time) + '.json'
            labels_ext = extract_labels(json_folder_path + json_file, start_time, duration_extract)
            create_label_json(path_json, labels_ext, start_time)

            positive_extracts += 1
            counts += 1

        elif labeled_prop == 0 and negative_extracts < nbre_extracts_neg:
            path_wav = path_database + 'negative/' + wav_file[:-4] + '_neg_t0_'+ str(start_time) + '.wav'
            indx_start = int(start_time * audio_fs)
            indx_end = int((duration_extract + start_time) * audio_fs) + 1
            wavfile.write(path_wav, audio_fs, audio_data[indx_start:indx_end])

            path_json = path_database + 'json/' + wav_file[:-4] + '_neg_t0_'+ str(start_time) + '.json'
            create_label_json(path_json)

            negative_extracts += 1
            counts += 1

        else:
            counts += 1

    return None


import librosa
import numpy as np
from scipy.io import wavfile
from scipy import signal
from math import *


def load_wav(path, decimate=None):
    """Load audio data.

        Parameters
        ----------
        path: str
            Wav file path.
        decimate: int
            If not None, downsampling by a factor of `decimate` value.

        Returns
        -------
        S: array-like
            Array of shape (Mel bands, time) containing the spectrogram.
    """
    fs, data = wavfile.read(path)

    data = data.astype(np.float32)

    if decimate is not None:
        data = signal.decimate(data, decimate)
        fs /= decimate

    return fs, data

def load_mp3(path):
    """Load audio data mp3 format.

        Parameters
        ----------
        path : str
            mp3 file path.

        Returns:
        -------
        data : array-like
            Audio data.
        fs : int
            Sampling frequency in Hz.
    """
    data, fs = librosa.core.load(path, sr=None)

    return fs, data


def load_audio(path):
    """Load audio data, mp3 or wav format

    Parameters
    ----------
        path : str
            audio file path.

    Returns:
    -------
        data : array-like
            Audio data.
        fs : int
            Sampling frequency in Hz.
    """
    if path[-4:] == ".wav":
        fs, data = load_wav(path)

    elif path[-4:] == ".mp3":
        fs, data = load_mp3(path)

    else:
        raise ValueError("Wrong file format, use mp3 or wav")

    return fs, data
  

def cut_audio(old_path, new_path, start, end):
    """
        Cut audio data to specific starting and end point and save it as a new wav file

        Parameters
        ----------
        old_path : str
            Original wav file path.
        new_path : str
            New wav file path.
        start : float
            Desired start time of new audio in seconds.
        end : float
            Desired end time of new audio in seconds.

    """
    fs, data = wavfile.read(old_path)
    indx_start = int(start*fs)
    indx_end = int(end*fs)+1
    wavfile.write(new_path,fs,data[indx_start:indx_end])

    return True


def create_spec(data, fs, n_mels=32, n_fft=2048, hop_len=1024):
    """Compute the Mel spectrogram from audio data.

        Parameters
        ----------
        data: array-like
            Audio data.
        fs: int
            Sampling frequency in Hz.
        n_mels: int
            Number of Mel bands to generate.
        n_fft: int
            Length of the FFT window.
        hop_len: int
            Number of samples between successive frames.

        Returns
        -------
        S: array-like
            Array of shape (Mel bands, time) containing the spectrogram.
    """
    # Calculate spectrogram
    S = librosa.feature.melspectrogram(
      data, sr=fs, n_fft=n_fft, hop_length=hop_len, n_mels=n_mels)
    S = S.astype(np.float32)

    # Convert power to dB
    S = librosa.power_to_db(S)

    return S


def wav2spc(wav_file, fs=44100, n_mels=40, n_fft=2048, hop_len=1024, duration=None):
    """Load a wav file and compute its MEL spectogram.

    Parameters
    ----------
    wav_file: str
        path to a wav file.
    fs: int
        Sampling frequency in Hz.
    n_mels: int
        Number of Mel bands to generate.
    n_fft: int
        Length of the FFT window.
    hop_len: int
        Number of samples between successive frames.
    duration: int
        Duration of the sound to consider (starting at the beginning)
        If None, no truncature is made

    Returns
    --------
    spec: array-like
        Array of shape (Mel bands, time) containing the spectrogram.
    """

    x_fs, x = load_wav(wav_file)
    if duration is not None:
        x = x[:int(x_fs * duration) + 1]

    if x_fs != fs:
        raise ValueError(f"wav file with wrong frequency {x_fs}: {wav_file}")
    spec = create_spec(x, fs, n_mels, n_fft, hop_len)
    return spec


def file2spec(path_file, scale_spec="linear", N_MELS=40, window_length=0.020, overlap=0.5, f_max=15000, duration=None):
    """ Compute spectrogram from a wav or mp3 file.

    Parameters
    ----------
        path_file : str
            path to a wav or mp3 file.
        scale_spec : str
            scale used to use to compute spectrogram, can be "linear" or "MEL".
        N_MELS : int
            Number of Mel bands to generate.
        window_length : float
            Length of the FFT window in seconds.
        overlap : float
            Overlap of the FFT windows.
        f_max : int
            Maximum frequency of the FFT domain.
        duration: int
            Duration of the sound to consider (starting at the beginning)
            If None, no cut is made

    Returns:
    -------
        spec : array-like
            Array of shape (frequency, time) containing the spectrogram.
        t : array-like
            Array of shape (time, 1) containing the time scale of spectogram.
            None if MEL scale is used
        f : array-like
            Array of shape (frequency, 1) containing the frequency scale of spectogram.
            None if MEL scale is used
    """

    # Load audio file
    x_fs, x = load_audio(path_file)

    shape = np.shape(x)
    # If the file contains several channel
    if len(shape) > 1:
        x = np.sum(x, axis=1)
    if duration is not None:
        x = x[:int(x_fs * duration) + 1]

    # Derive FFT parameters
    N_FFT = int(window_length * x_fs) + 1
    HOP_LEN = int(overlap * window_length * x_fs) + 1

    # Compute spectrograms
    if (scale_spec == "linear"):
        frequency_resolution = x_fs / N_FFT
        size_frequency_axis = 1 + floor(f_max / frequency_resolution)
        f, t, spec = signal.stft(x, fs=x_fs, nperseg=N_FFT, noverlap=HOP_LEN)
        # scipy returns a complex array, only the modulus is used in spectograms
        spec = np.abs(spec)
        # remove frequency above f_max
        if f[-1] > f_max:
            fsup_to_fmax = np.where(f > f_max)
            f = f[0:fsup_to_fmax[0][0] + 1]
            spec = spec[0:fsup_to_fmax[0][0] + 1, :]

    elif (scale_spec == "MEL"):
        # librosa library does not give access to t and f
        spec = librosa.feature.melspectrogram(x, sr=x_fs, n_fft=N_FFT, hop_length=HOP_LEN, n_mels=N_MELS)
        spec = np.abs(spec)
        t = None
        f = None

    else:
        raise ValueError(f"Wrong scale_spec parameter {scale_spec}, use linear or MEL")

    # Convert power to dB with the minimum as a reference, only positive dB
    spec = librosa.power_to_db(spec, ref=np.min(spec))

    return spec, t, f, x_fs



import matplotlib.pyplot as plt
import pylab
import numpy as np
import librosa.display

from microfaune import audio, labeling

def plot_spec(spec, t, f, fs, scale_spec="linear", window_length=0.2, overlap=0.5,
              plot_title="", fig_size=(20, 5), save_fig=False, save_path="spec.png", plot_main_frequencies=False):
    """ Plot spectrogram.ss
           Parameters
            ----------
            spec : array-like
                Array of shape (frequency, time) containing the spectrogram.
            t : array-like
                Array of shape (time, 1) containing the time scale of spectrogram.
                None if MEL scale is used
            f : array-like
                Array of shape (frequency, 1) containing the frequency scale of spectrogram.
                None if MEL scale is used
            fs : int
                Sampling frequency in Hz.
            scale_spec : str
                scale used to use to compute spectrogram, can be "linear" or "MEL".
            window_length : float
                Length of the FFT window in seconds.
            overlap : float
                Overlap of the FFT windows.
            plot_title : str
                Title of the plotted figure.
            fig_size : (int, int)
                Size of the plotted figure.
            save_fig : boolean
                True if the plot is saved, wrong otherwise.
            save_path : str
                Path of the saved figure.
            plot_main_frequencies : boolean
                True if median, minimum and maximum frequency are plotted on spectrograms, wrong otherwise.

            Returns:
            -------
            None
        """

    plt.set_cmap('inferno')
    pylab.rcParams['figure.figsize'] = fig_size
    plt.close()

    # Derive FFT parameters
    HOP_LEN = int(overlap * window_length * fs) + 1

    if scale_spec == "linear":
        plt.pcolormesh(t, f, spec)
        plt.xlabel('Time [sec]')
        plt.ylabel('Frequency [Hz]')

        if plot_main_frequencies:
            [freq_median, freq_min, freq_max] = find_most_used_frequencies(f, spec)

            plt.plot([0, t[-1]], [freq_min, freq_min], 'k')
            plt.plot([0, t[-1]], [freq_median, freq_median], 'w')
            plt.plot([0, t[-1]], [freq_max, freq_max], 'k')

    elif scale_spec == "MEL":
        librosa.display.specshow(spec, x_axis='time', y_axis='mel', sr=fs, hop_length=HOP_LEN)

    else:
        raise ValueError(f"Wrong scale_spec parameter {scale_spec}, use linear or MEL")

    plt.colorbar(format='%+2.0f dB')
    plt.title(plot_title)
    if save_fig:
        plt.savefig(save_path)
    plt.show()
    plt.close()

    return None


def find_most_used_frequencies(f, spec):
    """ Detect frequency used by the bird.

           Parameters
            ----------
            f : array-like
                Array of shape (frequency, 1) containing the frequency scale of spectrogram.
            spec : array-like
                Array of shape (frequency, time) containing the spectrogram.

            Returns:
            -------
            freq_median : int
                Median frequency of bird song in Hz.
            freq_min : int
                Minimum frequency of bird song in Hz (10% quantile).
            freq_max : int
                Maximum frequency of bird song in Hz (90% quantile).
        """

    # Removing lower frequencies often saturated
    f = f[4:]
    spec = spec[4:, :]

    quantile_95 = np.quantile(spec, 0.95)
    pixels_above_q95 = np.where(spec > quantile_95)
    freq_most_used = f[pixels_above_q95[0]]

    freq_median = round(np.quantile(freq_most_used, 0.50), 0)
    freq_min = round(np.quantile(freq_most_used, 0.10), 0)
    freq_max = round(np.quantile(freq_most_used, 0.90), 0)

    return [freq_median, freq_min, freq_max]


def plot_audio(fs, data):
    """
        Plot audio data.

        Parameters
        ----------
        data : array-like
            Audio data.
        fs : int
            Sampling frequency in Hz.

        Returns:
        -------
        None
    """
    plt.figure()
    sampled_points = len(data)
    time = [ti * 1. / fs for ti in range(sampled_points)]
    plt.plot(time, data)
    plt.xlabel('t [s]')
    plt.ylabel('P [W]')
    plt.show()
    plt.close()

    return None


def plot_charac_audio(json_file_path, audio_file_path):
    """ Plot the characteritic function derived on audio time scale from the labels in json file
                Parameters
                ----------
                json_file_path : str
                    Path of json file.
                audio_file_path : str
                    Path of audio file.

    """
    charac_func = labeling.charac_function_audio(json_file_path, audio_file_path)

    fs, data = audio.load_audio(audio_file_path)
    t_plot = np.array(range(len(data)))
    t_plot = t_plot / fs

    pylab.rcParams['figure.figsize'] = (20, 2)
    plt.close()

    plt.plot(t_plot, charac_func)
    plt.xlabel('time [s]')
    plt.ylabel('1 bird / 0 no bird')
    plt.title('Characteristic function on audio time scale')
    plt.show()

    return None


def plot_charac_spec(audio_file_path, window_length, overlap, charac_func_audio):
    """ Plot the characteristic function derived on spec time scale from the labels in json file
            Parameters
            ----------
            audio_file_path : str
                Path of audio file.
            window_length : float
                Length of the FFT window in seconds.
            overlap : float
                Overlap of the FFT windows.
            charac_func_audio : numpy array (nb bites in audio,1)
                Characteristic function derived in audio time scale, equal to 1 on labeled segments, 0 elsewhere

    """
    charac_func_spec = labeling.charac_function_spec(audio_file_path, window_length, overlap, charac_func_audio)

    fs, data = audio.load_audio(audio_file_path)
    t_plot = np.array(range(len(charac_func_spec)))
    t_plot = np.array(range(len(charac_func_spec))) * window_length * (1 - overlap)

    pylab.rcParams['figure.figsize'] = (20, 2)
    plt.close()

    plt.plot(t_plot, charac_func_spec)
    plt.xlabel('time [s]')
    plt.ylabel('1 bird / 0 no bird')
    plt.title('Characteristic function on spec time scale')
    plt.show()

    return None

def plot_charac_fs(fs, charac_func_fs):
    """ Plot the characteristic function with known sampling rate
        Parameters
        ----------
            fs: int
                Sampling frequency in Hz.
            charac_func_fs : numpy array (fs*duration, 1)
                Characteristic function derived with the desired sampling rate

    """
    duration = len(charac_func_fs) / fs
    t_fs = np.arange(0, duration, step=1./fs)

    plt.plot(t_fs, charac_func_fs)
    plt.xlabel('time [s]')
    plt.ylabel('1 bird / 0 no bird')
    plt.title('Characteristic function on with sampling rate ' + str(fs) + 'Hz')
    plt.show()

    return None


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct  5 16:18:32 2019


@author: christian
"""
import numpy as np
from keras.preprocessing.image import ImageDataGenerator
from microfaune import plot

class DataAugmentation:
    """Class to generate image data for rnn modeling
    """
    datagenerator_list = None

    def __init__(self, width_shift_range=None, horizontal_flip=True, \
                 brightness_range=None):
        """Initialization data generators

        Parameters
        ----------
        width_shift_range: list
            width of the random horizontal shift
        horizontal_flip: bool
            moving all pixels of the image horizontally,
            while keeping the image dimensions the same.
        brightness_range: list
        The brightness can be augmented by randomly darkening images,
        where has no effect on darkness

        Initiate
        -------
        datagenerator_list: list
            image data generators
        """
        if width_shift_range is None:
            width_shift_range = [-40, 40]
        if brightness_range is None:
            brightness_range = [0.4, 0.9]

        datagen_width_shift = ImageDataGenerator(width_shift_range=width_shift_range)
        datagen_horizontal_flip = ImageDataGenerator(horizontal_flip=horizontal_flip)
        datagen_brightness = ImageDataGenerator(brightness_range=brightness_range)
        self.datagenerator_list = [datagen_width_shift, datagen_horizontal_flip, datagen_brightness]

    def generate_augmentation(self, spec, y_val, my_range=5, to_display=False):
        """ data augmentation of one Spectrogram
        Parameters
        ----------
        spec spectogram
        y_val classification value

        Returns
        -------
        list_s
            list of Spectograms with Y list (duplicate from y input)
            the first Spectograms is the given input S
        list_y
            All y have the value of the given y
        """
        list_s = [spec]
        list_y = [y_val]
        for datagen in self.datagenerator_list:
            data = np.expand_dims(spec, axis=2)

            # expand dimension to one sample
            samples = np.expand_dims(data, 0)
            # prepare iterator
            cursor = datagen.flow(samples, batch_size=1)
            # generate samples and plot
            for _ in range(my_range):
                batch = cursor.next()
                # convert to unsigned integers for viewing
                image = batch[0].astype('uint8')
                image = image[:, :, 0]
                list_s.append(image)
                list_y.append(y_val)
                if to_display:
                    plot.plot_spec(image)
        return list_s, list_y

    def generate_augmentation_list(self, list_s, list_y, my_range=5, to_display=False):
        """ data augmentation of a  list of Spectrograms
        Parameters
        ----------
        list_s  vector of spectograms
        list_y  vector of y

        Returns
        -------
        list_s_augmented
            list of Spectograms augmented
        list_y_augmented
            list of y augmented with duplicate values
        """
        list_s_augmented = []
        list_y_augmented = []
        for spec, y_val in zip(list_s, list_y):
            lstx, lsty = self.generate_augmentation(spec, y_val, my_range, to_display)
            list_s_augmented += lstx
            list_y_augmented += lsty
        return list_s_augmented, list_y_augmented



from setuptools import setup, find_packages

packages = find_packages()
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='microfaune_ai',
    version='1.0.1',
    author="Microfaune - Data For Good",
    author_email="microfaune@fake.com",
    url='https://github.com/microfaune/microfaune_ai',
    download_url='https://github.com/microfaune/microfaune_ai/tree/main',
    description='Module package used for the Microfaune project',
    long_description='Biodiversity evaluation and monitoring is the first step toward its protection. The goal of the Microfaune project is to evaluate avifauna in Cité Internationale park (Paris, France) from audio recordings',
    license='Open license',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: Free For Educational Use',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Multimedia :: Sound/Audio :: Analysis'
    ],
    install_requires=requirements,

    packages=packages,
    package_data={'': ['data/*.h5']}
)
